/**
 * @typedef {Object} Node
 *
 * @property {number} id dataset id
 * @property {('regular'|'group'|'alias')} type
 * @property {string} [dataClass] like person, document, etc.
 * @property {string} [name]
 * @property {string} [label]
 *
 * @property {NodeVis} [vis] storage for visualization data
 */

/**
 * @typedef {Object} NodeVis
 * @property {boolean}  [vis.isUserDefined]
 * @property {number}   [vis.replacedBy] id of presentational node which replaced this node in aggregation
 * @property {number}   [vis.bundledIn] id of presentational node behind which this one is bundled
 * @property {number[]} [vis.bundledNodes] ids of nodes bundled in this node
 * @property {number[]} [vis.tags] id of tags assigned to the node
 */

/**
 * @typedef {Object} Link
 *
 * @property {number} id dataset id
 * @property {number} source id of end node
 * @property {number} target id of end node
 *
 * @property {boolean|object} [direction] boolean for singlelink, object for multilink
 * @property {boolean}  [direction.nonDirectional] specific for multilinks
 * @property {boolean}  [direction.sourceToTarget] specific for multilinks
 * @property {boolean}  [direction.targetToSource] specific for multilinks
 *
 * @property {LinkVis}   [vis] storage for visualization data
 */

/**
 * @typedef {Object} LinkVis
 * @property {boolean}  [vis.isUserDefined]
 * @property {boolean}  [vis.isMultilink] specific for multilinks
 * @property {number}   [vis.inMultilink] id of multilink which it belongs to
 * @property {number}   [vis.newSource] specific for singlelinks:
 * @property {number}   [vis.newTarget]
 * @property {boolean}  [vis.isRemoved]
 * @property {boolean}  [vis.isArchived] specific for multilinks: when link is inactive, waiting to be recovered when ungrouping
 * @property {boolean}  [vis.forceTransferred] override isTransferred calculation, set during bundling as it cannot be computed afterwards
 * @property {number[]} [vis.tags] id of tags assigned to the link
 */

export default {};
