/**
 * Automatically import all module files named with suffix '.store.js' and export them
 * as single module objects. Modules will be named according to file structure, however,
 * the first directory level will be ignored.
 */
const requireModule = require.context('.', true, /\.store\.js$/);
const modules = {};

requireModule.keys().forEach(filename => {
  // create the module name from fileName
  // remove the store.js extension and capitalize
  const moduleName = filename
    .replace(/^\.\//, '') // remove './' prefix
    .replace(/^.+?\//, '') // remove first directory from the path
    .replace(/(\.store\.js)/g, ''); // remove 'store.js' suffix

  console.log(`vuex importing module '${filename}' with name '${moduleName}'`);

  modules[moduleName] = requireModule(filename).default || requireModule(filename);
});

export default modules;
